// Copyright 2017 Budapest University of Technology and Economics (BME IK)
//
// This file is part of CIRCLE Cloud.
//
// CIRCLE is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// CIRCLE is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along
// with CIRCLE.  If not, see <http://www.gnu.org/licenses/>.


#ifndef OCCILIB_SESSION_H_
#define OCCILIB_SESSION_H_

#include <string>
#include <exception>
#include <stdexcept>

#include "restclient-cpp/connection.h"
#include "restclient-cpp/restclient.h"
#include "json.hpp"

namespace OcciClient {

    using std::string;

    enum class RequestType{Get, Post, Delete, Put};


    class OcciSession {
    private:

        RestClient::Connection* connection = nullptr;
        bool csrftokenRequired = false;
        string csrfuri;

        void setCsrfTokenHeader(){
            auto resp = this->connection->getLastResponse();
            string csrftoken;
            try {
                csrftoken = resp.cookies.at("csrftoken");
            }
            catch (const std::out_of_range& e) {
                return;
            }
            this->connection->AppendHeader("X-CSRFToken", csrftoken);
        }


    public:

        OcciSession(const char* url, bool insecure = false, bool csrf = false){
            RestClient::init();
            this->connection = new RestClient::Connection(url);
            if (insecure) {
                this->connection->SetHostVerify(false);
                this->connection->SetPeerVerify(false);
            }
            this->csrftokenRequired = csrf;
            RestClient::HeaderFields headers;
            headers["Accept"] = "application/json";
            headers["Content-type"] = "application/json";
            headers["Referer"] = url;
            this->connection->SetHeaders(headers);
        }

        ~OcciSession(){
            RestClient::disable();
        }

        nlohmann::json doRequest(string uri, RequestType type, nlohmann::json body = nullptr){
            RestClient::Response r;

            if (this->csrftokenRequired)
                this->setCsrfTokenHeader();

            if (type == RequestType::Post)
                r = connection->post(uri, body.dump());
            else if (type == RequestType::Put)
                r = connection->put(uri, body.dump());
            else if (type == RequestType::Delete)
                r = connection->del(uri);
            else
                r = connection->get(uri);
            nlohmann::json result;
            try {
                result = nlohmann::json::parse(r.body);
            }
            catch (std::invalid_argument& e) {
                result = "{}"_json;
                throw std::domain_error("Didn't get a nlohmann::json response from the OCCI server.");
            }
            try {
                throw std::logic_error(result["error"].get<string>());
            }
            catch (std::domain_error e) {
                return result;
            }
        }

        nlohmann::json get(string uri) {
            return doRequest(uri, RequestType::Get);
        }

        nlohmann::json post(string uri, nlohmann::json body = nullptr) {
            return doRequest(uri, RequestType::Post, body);
        }

        nlohmann::json put(string uri, nlohmann::json body = nullptr) {
            return doRequest(uri, RequestType::Put, body);
        }

        nlohmann::json del(string uri) {
            return doRequest(uri, RequestType::Delete);
        }

        void circleOcciLogin(string username, string password){
            get("login/");
            string body = "{\"username\": \"" + username + "\", \"password\": \"" + password + "\"}";
            post("login/", nlohmann::json::parse(body));
        }

        nlohmann::json queryInterface(){
            return get("-/");
        }

    };

}

#endif // OCCILIB_SESSION_H_
